#if !defined(TREE_H)
#define TREE_H


/* includes *******************************************************************/

#include "TreeOpener.h"


/* part from original file ****************************************************/

#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGGElement.hpp>
#include <w3c/svg/SVGUseElement.hpp>
#include <w3c/svg/SVGRectElement.hpp>

#include <hsm/interaction/Protocol.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>


/* foraward declarations ******************************************************/

class Node;


/* ANode class definition *****************************************************/

class ANode : public hsm::SVGLBehaviour {
public :
	Node *parent;

public :
	ANode(svg::SVGElement *e, hsm::SVGLWindow *w);

	virtual unsigned int getHeight();
	void translateAllBrothers(int dy);

private :
	void translateBrothers(int dy);
};


/* Leaf class definition ******************************************************/

class Leaf : public ANode, public hsm::PushPop {
	static const hsm::SVGLBehaviour::Factory::Register< Leaf > registration;

public :
	Leaf(svg::SVGElement *e, hsm::SVGLWindow *w);
	virtual ~Leaf();
};

class TreeOpener;


/* Node class definition ******************************************************/

class Node : public ANode {
	friend class TreeOpener;

private :
	svg::SVGRectElement *background;
	svg::SVGUseElement *handle;
	svg::SVGGElement *content;

	static const hsm::SVGLBehaviour::Factory::Register< Node > registration;

public : 
	bool isOpen;
	
public :
	Node(svg::SVGElement *e, hsm::SVGLWindow *w);
	virtual ~Node();

	void open();
	void close();

	void resizeBackground(int dHeight);
	virtual unsigned int getHeight();
	
	void add(svg::SVGElement *e, unsigned int p);
	void remove(svg::SVGElement *e);
};

#endif // !defined(TREE_H)
