////////////////////////////////////////////////////////////////////////////////
// File    : simple.cpp
// Content : a simple program to test interactions
// History : 09-Jul-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/system/plugin/PlugIn.h>

#include <svgl/Parser.hpp>

#include <w3c/svg/SVGSVGElement.hpp>

#include <interactions.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "simple - a simple program to test interactions." << endl << flush;
	atexit(quitHandler);
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("window");
	window->setTitle("simple");
	
	hsm::EventHandler *pointer = hsm::repository::getEventHandler("Pointer");
	window->addEventHandler(pointer);
	hsm::EventHandler *keyboard = hsm::repository::getEventHandler("Keyboard");
	window->addEventHandler(keyboard);
	

	// creating instruments
	// version avec multiplexage sur les boutons de la souris
//	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
//	Zoomer::Hsm *zoomer = new Zoomer::Hsm(Zoomer::Data(window, *pointer/"buttons"/hsm::ButtonPad::R, *pointer/"position"));

	// version avec multiplexage utilisant le shift
//	ShiftMultiplex::Hsm *shift = new ShiftMultiplex::Hsm(ShiftMultiplex::Data(window, *keyboard/"modifiers"/hsm::ButtonPad::SHIFT, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));

	// version qui cycle entre pan / zoom / pan and zoom multiplexés sur le shift
	Switcher::Hsm *switcher = new Switcher::Hsm(Switcher::Data(window, *keyboard/"keys"/' ', *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position", *keyboard/"modifiers"/hsm::ButtonPad::SHIFT));
	
	// creating the svg document
	svgl::Parser parser;
	std::stringstream strstr;
	strstr << "<?xml version='1.0' encoding='iso-8859-1'?>"
		"<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 03December 1999//EN' 'http://www.w3.org/Graphics/SVG/SVG-19991203.dtd'>"
		"<svg>  <text x='10' y='40' style='font-size:30; fill:black'> coucou </text> </svg>";
	svg::SVGDocument *document = parser.parseFromStream(&strstr);
	
	svg::SVGSVGElement *element = const_cast< svg::SVGSVGElement * >(document->getRootElement());
	window->setSize((int)(element->getWidth().getValue()), (int)(element->getHeight().getValue()));
	window->svgContent->appendChild(element);
	

	// full content pan & zoomable
	window->svgContent->setAttribute(unicode::String::createString("hsm:behaviour"), unicode::String::createString("panAndZoomable"));
	window->initBehaviours();
	

	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
//	delete translater;
//	delete zoomer;
	
//	delete shift;
	delete switcher;
	
	delete window;
	
	
	return EXIT_SUCCESS;
}
