////////////////////////////////////////////////////////////////////////////////
// File    : svgEditor.cpp
// Content : a simple svg editor
// History : 02-Oct-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <svgl/Parser.hpp>

#include <w3c/svg/SVGSVGElement.hpp>
	
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/svgl/SVGLCursor.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/Point.h>
#include <hsm/component/ButtonPad.h>
#include <hsm/system/plugin/PlugIn.h>

#include <interactions.h>

#include <hsm/svgl/Svgl.h>


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "svgEditor - svg editor using svgl & hsm." << endl << flush;
	atexit(quitHandler);
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("window");
	window->setTitle("svgEditor");

	hsm::EventHandler *pointer = hsm::repository::getEventHandler("Pointer");
	window->addEventHandler(pointer);
	hsm::EventHandler *keyboard = hsm::repository::getEventHandler("Keyboard");
	window->addEventHandler(keyboard);

	// creating instruments
	Translater::Hsm *translater = new Translater::Hsm(Translater::Data(window, *pointer/"buttons"/hsm::ButtonPad::L, *pointer/"position"));
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Actioner::Hsm *actioner = new Actioner::Hsm(Actioner::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));
	
	TabletManager::Hsm *tabletManager = new TabletManager::Hsm(TabletManager::Data(window));
	
	// adding the palette
	hsm::Point palettePosition(2);
	palettePosition = 20, 20;
	palettePosition += *window/"position"/hsm::Point::cast;
	std::string path;
	char *cep = getenv("EXE_PATH");
	if(cep != 0) path = cep;
	hsm::SVGLCursor *palette = new hsm::SVGLCursor(palettePosition, window, path + "/palette.svg", window->svgFeedback);		

	
	// getting the svg document
	svgl::Parser parser;
	svg::SVGDocument *document = 0;
	if(argc > 1) document = parser.parse(argv[1]);
	if(document != 0) {
		svg::SVGSVGElement *element = const_cast< svg::SVGSVGElement * >(document->getRootElement());
		window->setSize((int)(element->getWidth().getValue()), (int)(element->getHeight().getValue()));
		window->svgContent->appendChild(element);
	}
	
	window->initBehaviours();

	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	


	// clean-up
	delete translater;
	delete enterLeaver;
	delete actioner;
	
	delete tabletManager;

	delete palette;
	delete window;
	

	return EXIT_SUCCESS;
}