////////////////////////////////////////////////////////////////////////////////
// File    : widgets.cpp
// Content : a simple program to test svgl widgets
// History : 24-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
 Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>
 
 This program is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */


/* includes *******************************************************************/

#include <interactions.h>

#include <cstdlib>
#include <iostream>
#include <sstream>
using namespace std;

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/system/window/EventHandler.h>
#include <hsm/system/event/EventLoop.h>
#include <hsm/component/ButtonPad.h>

#include <svgl/Parser.hpp>

#include <w3c/svg/SVGSVGElement.hpp>


/* button callback ************************************************************/

void buttonCb() { cout << "Hello " << flush; }
void crossCb() { cout << "world" << endl; }


/* handle quit ****************************************************************/

void quitHandler() {
	cout << endl << "bye." << endl;
}


/* main ***********************************************************************/

int main(int argc, const char *argv[]) {
	cout << "widgets - a simple program to test svgl widgets." << endl << flush;
	atexit(quitHandler);
	
	// setting up the window and the pointer
	hsm::SVGLWindow *window = new hsm::impl::Component< hsm::SVGLWindow, hsm::processing::MonoAsync >("window");
	window->setTitle("widgets");
	
	hsm::EventHandler *pointer = hsm::repository::getEventHandler("Pointer");
	window->addEventHandler(pointer);
	
	// creating instruments
	EnterLeaver::Hsm *enterLeaver = new EnterLeaver::Hsm(EnterLeaver::Data(window, *pointer/"position"));
	Actioner::Hsm *actioner = new Actioner::Hsm(Actioner::Data(window, *pointer/"position"/hsm::Point::cast, *pointer/"buttons"/hsm::ButtonPad::L));


	// registering button callback
	hsm::Functor::add("buttonCb", hsm::Functor::create(&buttonCb));
	hsm::Functor::add("crossCb", hsm::Functor::create(&crossCb));
	
	
	// creating the svg document
	svgl::Parser parser;
	std::stringstream strstr;
	strstr << "<?xml version='1.0' encoding='iso-8859-1'?>"
		"<!DOCTYPE svg PUBLIC '-//W3C//DTD SVG 03December 1999//EN' 'http://www.w3.org/Graphics/SVG/SVG-19991203.dtd'>"
		"<svg xmlns:hsm='http://hsm.insitu.lri.fr/'"
		"     xmlns:hsm-arg='http://hsm.insitu.lri.fr/'"
		"     xmlns:xlink='http://www.w3.org/1999/xlink'>"
		"	<defs>" 
		"		<linearGradient id='popped' gradientUnits='objectBoundingBox' x1='0' y1='0' x2='0' y2='1'>"
		"			<stop offset='0' style='stop-color:white' />"
		"			<stop offset='1' style='stop-color:lightgray' />"
		"		</linearGradient>"
		"		<linearGradient id='pushed' gradientUnits='objectBoundingBox' x1='0' y1='0' x2='0' y2='1'>"
		"			<stop offset='0' style='stop-color:lightgray' />"
		"			<stop offset='1' style='stop-color:white' />"
		"		</linearGradient>"
		"		<symbol id='v'>"
		"			<path d='M -2 -1 h 4 l -2 2 z'/>"
		"		</symbol>"
		"		<symbol id='^'>"
		"			<path d='M -2 1 h 4 l -2 -2 z'/>"
		"		</symbol>"
		"	</defs>"
		
		

		// button definition
		"	<g hsm:behaviour='button' hsm-arg:callbackId='buttonCb' style='font-size:10; font-family:Verdana; font-weight: bold' transform='translate(20,20)'>"
		"		<g>"
		"			<rect style='fill:url(#pushed); stroke:gray' width='80' height='20' rx='4' ry='4'/>"
		"		</g>"
		"		<g>"
		"			<rect style='fill:url(#popped); stroke:gray' width='80' height='20' rx='4' ry='4'/>"
		"		</g>"
		"     <text x='20' y='13'>Hello</text>"
		"	</g>"
		
		// cross button definition
		"	<g hsm:behaviour='cross' hsm-arg:callbackId='crossCb' transform='translate(60,60)'>"
		"		<text x='-18' y='13'>World</text>"
		"		<g>"
		"			<use xlink:href='#v'/>"
		"			<use xlink:href='#^' y='18'/>"
		"		</g>"
		"		<rect style='fill:black; opacity:0' width='10' height='18' x='-10'/>"
		"		<rect style='fill:black; opacity:0' width='10' height='18' />"
		"	</g>"
		
//		//	hierarchical menu definition
//		"	<g hsm:behaviour='menu' transform='translate(120,20)' style='fill:white; stroke:gray' >"
//		"		<g>"
//		"			<rect width='40' height='20'/>"
//		"        <text x='5' y='13' fill='black' >File</text>"
//		"		</g>"
//		"		<g>"
//		"			<g hsm:behaviour='menu' transform='translate(0,20)' style='fill:white'>"
//		"				<g>"
//		"					<rect width='40' height='20'/>"
//		"					<text x='5' y='13' fill='black' >Open</text>"
//		"				</g>"
//		"				<g>"
//		"					<g hsm:behaviour='menu' transform='translate(40,0)' style='fill:white'>"
//		"						<g>"
//		"							<rect width='40' height='20'/>"
//		"							<text x='5' y='13' fill='black' >open 1</text>"
//		"						</g>"
//		"					</g>"
//		"					<g hsm:behaviour='menu' transform='translate(40,20)' style='fill:white'>"
//		"						<g>"
//		"							<rect width='40' height='20'/>"
//		"							<text x='5' y='13' fill='black' >open 2</text>"
//		"						</g>"
//		"					</g>"
//		"				</g>"
//		"			</g>"
//		"			<g hsm:behaviour='menu' transform='translate(0,40)' style='fill:white'>"
//		"				<g>"
//		"					<rect width='40' height='20'/>"
//		"					<text x='5' y='13' fill='black' >Close</text>"
//		"				</g>"
//		"			</g>"
//		"		</g>"
//		"	</g>"
		
		"</svg>";
	svg::SVGDocument *document = parser.parseFromStream(&strstr);
	
	svg::SVGSVGElement *element = const_cast< svg::SVGSVGElement * >(document->getRootElement());
	window->setSize((int)(element->getWidth().getValue()), (int)(element->getHeight().getValue()));
	window->svgContent->appendChild(element);
	
	window->initBehaviours();
	

	// launching interaction
	window->show(true);
	hsm::eventloop::loop();	
	
	
	// clean-up
	delete enterLeaver;
	delete actioner;
	
	delete window;
	
	
	return EXIT_SUCCESS;
}
