 ////////////////////////////////////////////////////////////////////////////////
// File    : SVGLBehaviour.cpp
// Content : classes for supporting interaction with svgl objects
// History : 25-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/Svgl.h>

#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGTransforms.hpp>
#include <w3c/svg/SVGTransformable.hpp>


namespace hsm {

/* SVGLBehaviour class implementation ******************************************/

	SVGLBehaviour::SVGLBehaviour(svg::SVGElement *e, SVGLWindow *w) : elem(e), window(w) {
		(*SVGLBehaviour::Factory::behaviours)[elem] = this;	
	}
	
	SVGLBehaviour::~SVGLBehaviour() {
		SVGLBehaviour::Factory::behaviours->erase(elem);	
	}
	
	
/* SVGLEnterLeave class implementation ****************************************/

	SVGLEnterLeave::SVGLEnterLeave(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		stylable = dynamic_cast< svg::SVGStylable * >(elem);
	}

	void SVGLEnterLeave::enter() {
		if(stylable != 0) {
			window->lock();
			strokeWidth = stylable->getStrokeWidth().getValue();
			stylable->setStrokeWidth(2 * strokeWidth);
			window->unlock();
		}
	}
	
	void SVGLEnterLeave::leave() {
		if(stylable != 0) {
			window->lock();
			stylable->setStrokeWidth(strokeWidth);	
			window->unlock();
		}
	}


/* SVGLTranslate class implementation *****************************************/

	SVGLTranslate::SVGLTranslate(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		svg::SVGTransformable *transformable = dynamic_cast< svg::SVGTransformable * >(elem);
		trans = transformable->getTranslate(true);
	}

	void SVGLTranslate::translate(const Point &p) {
		window->lock();
		svgl::getGlobalTransform(elem, sx, sy, tx, ty);
		trans->tx += p/Point::X / sx;
		trans->ty += p/Point::Y / sy;
		window->unlock();
	}


/* SVGLZoom class implementation **********************************************/

	SVGLZoom::SVGLZoom(svg::SVGElement *e, SVGLWindow *w) : SVGLBehaviour(e, w) {
		svg::SVGTransformable *transformable = dynamic_cast< svg::SVGTransformable * >(elem);
		scale = transformable->getScale(true);
		trans = transformable->getTranslate(true);
	}
	
	void SVGLZoom::zoom(float ds /* = 1. */) {
		ds = exp(ds * 0.01);
		window->lock();
		svgl::getGlobalTransform(elem, sx, sy, tx, ty);
		scale->sx *= ds;
		scale->sy *= ds;
		ds -= 1;
		trans->tx += ((tx - pick/Point::X) / sx + trans->tx) * ds;
		trans->ty += ((ty - pick/Point::Y) / sy + trans->ty) * ds;
		window->unlock();
	}


} // namespace hsm
