////////////////////////////////////////////////////////////////////////////////
// File    : SVGLBehaviourFactory.cpp
// Content : classes for supporting interaction with svgl objects
// History : 25-Aug-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <hsm/system/plugin/PlugIn.h>
#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/svgl/Svgl.h>

#include <w3c/svg/SVGDocument.hpp>
#include <w3c/svg/SVGElement.hpp>


namespace hsm {
		
/* SVGLBehaviour::Factory class implementation ********************************/

	SVGLBehaviour::Factory::ConstructorMap *SVGLBehaviour::Factory::constructors = 0;
	SVGLBehaviour::Factory::BehaviourMap *SVGLBehaviour::Factory::behaviours = 0;
	const SVGLBehaviour::Factory *SVGLBehaviour::Factory::instance = 0;
	
	SVGLBehaviour::Factory::Factory() {
		constructors = new ConstructorMap;
		behaviours = new BehaviourMap;
	}
	
	void SVGLBehaviour::Factory::ensureLoading() {
		if(instance == 0) instance = new Factory();
	}
	
	SVGLBehaviour *SVGLBehaviour::Factory::create(std::string type, svg::SVGElement *e, SVGLWindow *w) {
		ConstructorMap::iterator constructor = constructors->find(type);
		ConstructorMap::iterator end = constructors->end();
			
		// do we already know the interaction ?
		if(constructor != end) {
			return (*constructor).second(e, w);
		}
			
		// do we have a plugin ?
		if(hsm::plugin::load("behaviours/" + type)) {
			constructor = constructors->find(type);
			return (*constructor).second(e, w);
		}

		return 0;
	}
	
	SVGLBehaviour *SVGLBehaviour::Factory::get(svg::SVGElement *e) {
		BehaviourMap::iterator behaviour = behaviours->find(e);
		if(behaviour != behaviours->end()) {
			return behaviour->second;
		} else {
			return 0;
		}
	}


} // namespace hsm
