////////////////////////////////////////////////////////////////////////////////
// File    : SVGLCursor.cpp
// Content : Cursor class
// History : 28-Jan-2004 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo ////////////////////////////////////////////////////////////////////////
// + there is a bug in svgl on image translation (the translation is done twice)
//   -> the workaround consist in including the image inside a group element and
//   translating the group.
// - remove svgl thing in the destructor
// - check if there is at least one g element in the svg document


/* includes *******************************************************************/

#include <SVGLCursor.h>

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/util/Debug.h>

#include <w3c/svg/SVGTransforms.hpp>

#include <svgl/Parser.hpp>

	
namespace hsm {

/* SVGLCursor class implementation ********************************************/

	SVGLCursor::SVGLCursor(Point &p, SVGLWindow *w, const std::string &name, svg::SVGGElement *l /* = 0 */, float offsetX /* = 0. */, float offsetY /* = 0. */)
	 : impl::Component< Component, processing::MonoAsync >(), position(&p), window(w), windowPosition(2), layer((l == 0) ? w->svgCursors : l), cursor(0) {

		svgl::Parser parser;
		svg::SVGDocument *document = parser.parse(name.c_str());

		dom::Element *element = document->getElementById(unicode::String::createString("cursor"));
		cursor = dynamic_cast< svg::SVGGElement * >(element);

		if(cursor == 0) {
			HSM_ERRPUT("Unable to construct hsm::SVGLCursor from file : " << name)
			
		} else {
			window->lock();
			layer->appendChild(cursor);
			window->instanciateBehaviour(cursor);
			
			trans = cursor->getTranslate(true);
			hotX = trans->tx + offsetX;
			hotY = trans->ty + offsetY;
			
			window->screen2window(*position, windowPosition);
			trans->tx = windowPosition/Point::X + hotX;
			trans->ty = windowPosition/Point::Y + hotY;
			
			window->unlock();
			position->addReceiver(this);
		}
	}
	
	SVGLCursor::~SVGLCursor() {
		// remove svgl things
		if(cursor != 0) {
			position->removeReceiver(this);
			window->lock();
			layer->removeChild(cursor);
			window->unlock();
		}
	}
	
	void SVGLCursor::setPosition(Point &p) {
		if(cursor != 0) {
			position->removeReceiver(this);
			position = &p;
			position->addReceiver(this);
		}
	}
	
	void SVGLCursor::process(const Event &event) {
		if(event.message != Event::DELETED) {
			window->lock();
			window->screen2window(*position, windowPosition);
			trans->tx = windowPosition/Point::X + hotX;
			trans->ty = windowPosition/Point::Y + hotY;
			window->unlock();
		}
	}
	
	
} // namespace hsm
