////////////////////////////////////////////////////////////////////////////////
// File    : SVGLWindow.h
// Content : Window class with svgl support
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#if !defined(HSM_SVGLWINDOW_H)
#define HSM_SVGLWINDOW_H


/* includes *******************************************************************/

#include <hsm/system/window/GLWindow.h>

#include <w3c/svg/SVGDocument.hpp>
#include <w3c/svg/SVGSVGElement.hpp>
#include <w3c/svg/SVGElement.hpp>
#include <w3c/svg/SVGGElement.hpp>


/* forward declarations ******************************************************/

namespace svgl {
	class DisplayManager;
	class PickManager;
}

namespace svg {
	class SVGElement;
}
	
namespace hsm {

	class Point;
	

/* SVGLWindow class definition ************************************************/

	class SVGLWindow : public GLWindow {
	protected :
		svgl::DisplayManager *svglDisplayManager;
		svgl::PickManager    *svglPickManager;
		
	public :
		svg::SVGDocument   *svgDocument;
		svg::SVGSVGElement *svgRoot;
		svg::SVGGElement   *svgCursors;
		svg::SVGGElement   *svgFeedback;
		svg::SVGGElement   *svgContent;
		
	public :
		         SVGLWindow(const Id &id = Id());
		virtual ~SVGLWindow();
		
	protected :
		virtual void init();
		virtual void display();
		virtual void reshape(unsigned int width, unsigned int height);

	public :
		template< class P >
		P *pick(const Point &p, const svg::SVGElement *start = 0, bool topOnly = true);
		
		void initBehaviours();
		void instanciateBehaviour(svg::SVGElement *elem);
	};


} // namespace hsm


#endif // !defined(HSM_SVGLWINDOW_H)