////////////////////////////////////////////////////////////////////////////////
// File    : SVGLWindowPicking.h
// Content : Window class with svgl support (picking implementation)
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


#if !defined(HSM_SVGLWINDOWPICKING_H)
#define HSM_SVGLWINDOWPICKING_H


/* includes *******************************************************************/

#include <hsm/svgl/SVGLWindow.h>
#include <hsm/svgl/SVGLBehaviour.h>
#include <hsm/interaction/Protocol.h>

#include <w3c/svg/SVGDocument.hpp>
#include <w3c/svg/SVGElement.hpp>

#include <svgl/PickManager.hpp>

	
namespace hsm {

/* SVGLWindow::pick implementation ********************************************/

	template< class B >
	B *SVGLWindow::pick(const Point &p, const svg::SVGElement *start /* = 0 */, bool topOnly /* = true */) {
		COMPILE_ASSERT(BASE_OF(hsm::Protocol, B), B_should_inherite_from_Protocol);

		Point wp(2);
		screen2window(p, wp);

		bool cut = (start != 0);

		lock();
		os::makeCurrent(this);
		svgRoot->removeChild(svgCursors);
		svgl::PickManager::PickResult pickResult = svglPickManager->pick(svgDocument, wp/Point::X, wp/Point::Y, 2, 2);
		svgRoot->appendChild(svgCursors);
		unlock();
		
		svgl::PickManager::ReversePickResult hitIterator = svglPickManager->reversePick();
		
		for(; hitIterator.first != hitIterator.second; ++hitIterator.first) {
			svgl::PickManager::HitIterator::value_type svgElementIterator = *hitIterator.first;
			
			// searching the starting point of the picking (just return elements in branchs located under start in z order)
			if(cut) {
				for(; svgElementIterator.first != svgElementIterator.second; ) {
					--svgElementIterator.second;
					if(*svgElementIterator.second != start) continue;
					cut = false;
					break;
				}
				
			// searching the inner most element implementing the requested protocol
			} else {
				for(; svgElementIterator.first != svgElementIterator.second; ) {
					--svgElementIterator.second;
					svg::SVGElement *elem = *svgElementIterator.second;
					SVGLBehaviour *behaviour = SVGLBehaviour::Factory::get(elem);
					B *protocol = dynamic_cast< B * >(behaviour);
					if(protocol != 0) {
						protocol->setPickPoint(wp);
						return protocol;
					}
				}
				if(topOnly) { break; }
			}
		}
			
		return 0;
	}


} // namespace hsm


#endif // !defined(HSM_SVGLWINDOWPICKING_H)