////////////////////////////////////////////////////////////////////////////////
// File    : Selector.MACOSX.cpp
// Content : A manager for select operations
// History : 31-Jan-2003 - [rb] creation
//           08-Feb-2003 - [rb] rewrite, added add / remove
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Selector.h>

#include <hsm/system/select/Alert.h>
#include <hsm/system/thread/Mutex.h>
#include <hsm/core/Event.h>
#include <hsm/core/Component.h>


namespace hsm {

/* Selector implementation ****************************************************/

	void Selector::os::watch(void *selector) {
		Selector *s = reinterpret_cast< Selector * >(selector);

		fd_set rFdSet, wFdSet, eFdSet;
		FD_ZERO(&rFdSet); FD_ZERO(&wFdSet); FD_ZERO(&eFdSet);

		file::Descriptor maxFd(*(s->alert));
		FD_SET(maxFd, &rFdSet);

		s->mutex->lock();
		for(StatusMapMap::const_iterator smmi = s->statusMapMap.begin(); smmi != s->statusMapMap.end(); ++smmi) {
			file::Descriptor fd = (*smmi).first;
			const StatusMap &sm = (*smmi).second;
			if(maxFd < fd) maxFd = fd;
			for(StatusMap::const_iterator smi = sm.begin(); smi != sm.end(); ++smi) {
				Status stat  = (*smi).second;
				if(stat & R) FD_SET(fd, &rFdSet);
				if(stat & W) FD_SET(fd, &wFdSet);
				if(stat & E) FD_SET(fd, &eFdSet);
			}
		}
		s->mutex->unlock();

		select(maxFd + 1, &rFdSet, &wFdSet, &eFdSet, 0);

		if(FD_ISSET(*(s->alert), &rFdSet)){
			s->alert->reset();
		}
		
		s->mutex->lock();
		for(file::Descriptor fd = 0; fd <= maxFd; ++fd) {
			Status stat = Status(0);
			if(FD_ISSET(fd, &rFdSet)) stat = Status(stat | R);
			if(FD_ISSET(fd, &wFdSet)) stat = Status(stat | W);
			if(FD_ISSET(fd, &eFdSet)) stat = Status(stat | E);
			if(stat > 0) {
				StatusMap &sm = s->statusMapMap[fd];
				for(StatusMap::const_iterator smi = sm.begin(); smi != sm.end(); ++smi) {
					Functor *f = (*smi).first;
					Status status = (*smi).second;
					if(stat & status) (*f)(new Event(0, stat, fd));
				}
			}
		}
		s->mutex->unlock();
	}


} // namespace hsm