////////////////////////////////////////////////////////////////////////////////
// File    : GLWindow.MACOSX.cpp
// Content : GLWindow::os struct implementation
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <GLWindow.h>

#include <cstdlib> // getenv
#include <string>
#include <sstream>


namespace hsm {

/* GLWindow::os struct implementation *****************************************/

	GLWindow::os::Context GLWindow::os::createGlContext(GLWindow *glw) {
		SetPortWindowPort(glw->window);

		unsigned int next = 0;
		GLint attrib[] = { AGL_NONE, AGL_NONE, AGL_NONE, AGL_NONE,
		                   AGL_NONE, AGL_NONE, AGL_NONE, AGL_NONE,
		                   AGL_NONE, AGL_NONE, AGL_NONE, AGL_NONE,
		                   AGL_NONE, AGL_NONE, AGL_NONE, AGL_NONE };
		
		// default setup
		attrib[next++] = AGL_MP_SAFE;
		attrib[next++] = AGL_NO_RECOVERY;		
		attrib[next++] = AGL_RGBA;
		attrib[next++] = AGL_DOUBLEBUFFER;
		attrib[next++] = AGL_PIXEL_SIZE;   attrib[next++] = 32;
		attrib[next++] = AGL_STENCIL_SIZE; attrib[next++] = 8;
		attrib[next++] = AGL_DEPTH_SIZE;   attrib[next++] = 32;
		
		// test for multisample
		char *cgs = getenv("HSM_GL_SAMPLES");
		unsigned int gs = 0;
		if(cgs != 0) {
			std::string sgs = cgs;
			std::stringstream strgs; strgs << sgs;
			strgs >> gs;
			attrib[next++] = AGL_SAMPLE_BUFFERS_ARB; attrib[next++] = 1;
			attrib[next++] = AGL_SAMPLES_ARB;        attrib[next++] = gs;
		}
		
		// creating the openGl context
		AGLPixelFormat apf = aglChoosePixelFormat(0, 0, attrib);
		Context glContext = aglCreateContext(apf, 0);
		aglDestroyPixelFormat(apf);

		return glContext;
	}

	void GLWindow::os::destroyGlContext(GLWindow *glw) {
		aglSetCurrentContext(0);
		aglSetDrawable(glw->glContext, 0);
		aglDestroyContext(glw->glContext);		
	}
	
	void GLWindow::os::makeCurrent(GLWindow *glw) {
		aglSetDrawable(glw->glContext, GetWindowPort(glw->window));
		aglSetCurrentContext(glw->glContext);
	}
	
	void GLWindow::os::swapBuffers(GLWindow *glw) {
		aglSwapBuffers(glw->glContext);	
	}

	void GLWindow::os::setVBLSync(GLWindow *glw, long vbl /* = 1 */) {
		aglSetCurrentContext(glw->glContext);
		CGLSetParameter(CGLGetCurrentContext(), kCGLCPSwapInterval, &vbl);	
	}

	void GLWindow::os::reshape(GLWindow *glw) {
		aglUpdateContext(glw->glContext);	
	}

} // namespace hsm
