////////////////////////////////////////////////////////////////////////////////
// File    : Window.cpp
// Content : Window class
// History : 11-Oct-2001 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Window.h>

#include <hsm/util/Debug.h>
#include <hsm/component/Point.h>


namespace hsm {

/* Window class implementation ************************************************/

	Window::Window(const Id &id /* = Id() */) : Composite(id) {
		HSM_PRINT(1, "[Window::Window()]")
		add(position = new Point("position", 2));
		add(size = new Point("size", 2));

		*position = 100, 100;
		*size = 640, 480;
		
		window = os::create(this, "", *position/Point::X, *position/Point::Y, *size/Point::X, *size/Point::Y);
	}

	Window::~Window() {
		HSM_PRINT(1, "[Window::~Window()]")
		os::destroy(window);
	}

	void Window::setPosition(const Point &p) {
		setPosition(p/Point::X, p/Point::Y);
	}

	void Window::setPosition(int x, int y) {
		os::setPosition(window, x, y, *size/Point::X, *size/Point::Y);
	}

	void Window::setSize(const Point &s) {
		setSize(s/Point::X, s/Point::Y);
	}

	void Window::setSize(int w, int h) {
		os::setSize(window, *position/Point::X, *position/Point::Y, w, h);
	}

	void Window::setTitle(const std::string &title) {
		os::setTitle(window, title);
	}

	void Window::show(bool s /* = true */) {
		if(s) {
			os::show(window);
		} else {
			os::hide(window);
		}
	}
	
	void Window::hide() {
		show(false);
	}
	
	void Window::zoom(bool s /* = true */) {
		if(s) {
			os::zoom(window);
		} else {
			os::unzoom(window);
		}
	}
	
	void Window::showCursor(bool s /* = true */) const {
		if(s) {
			os::showCursor();
		} else {
			os::hideCursor();
		}
	}
	
	void Window::hideCursor() const {
		showCursor(false);
	}
	
	void Window::screen2window(const Point &sp, Point &wp) const {
		wp = sp;
		wp -= *position;
	}

	void Window::window2screen(const Point &wp, Point &sp) const {
		sp = wp;
		sp += *position;
	}


	void Window::addEventHandler(const EventHandler *eh) {
		os::addHandler(this, eh);
	}
	
	void Window::removeEventHandler(const EventHandler *eh) {
		os::removeHandler(this, eh);
	}


} // namespace hsm
