////////////////////////////////////////////////////////////////////////////////
// File    : Functor.cpp
// Content : class prototype for a function object
// History : 18-Jan-2002 - [rb] creation
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


/* includes *******************************************************************/

#include <Functor.h>


namespace hsm {

/* static member initialisation ***********************************************/

	Functor::FunctorMap Functor::functors;
	
	
/* helper functions ***********************************************************/

	Functor *Functor::create(void (*pf)()) {
		return new FunctionFunctor(pf);
	}

	Functor *Functor::create(void (*pf)(void *)) {
		return new FunctionPVoidFunctor(pf);
	}

	void Functor::add(const std::string &id, Functor *fn) {
		functors[id] = fn;
	}
	
	void Functor::erase(const std::string &id) {
		functors.erase(id);
	}
	
	Functor *Functor::get(const std::string &id) {
		FunctorMap::const_iterator i = functors.find(id);
		if(i != functors.end()) {
			return (*i).second;
		}
		return 0;
	}
		
	void Functor::execute(const std::string &id, void *arg /* = 0 */) {
		Functor *fn = get(id);
		if(fn != 0) {
			(*fn)(arg);
		}
	}


} // namespace hsm
