////////////////////////////////////////////////////////////////////////////////
// File    : Pointer.MACOSX.cpp
// Content : hsm system pointer component
// History : 18-Oct-2001 - [rb] creation
//           17-Mar-2003 - [rb] fixed a bug in MacOS X version (the dx and dy
//                              when using a tablet takes into acount the
//                              tablet movment)
////////////////////////////////////////////////////////////////////////////////

/*
    Copyright (C) 2004  Renaud Blanch <rb.hsm@laposte.net>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/


// ToDo ////////////////////////////////////////////////////////////////////////
// - allocate parts with physical caracteristics of the mouse

// Hack [MacOSX] ///////////////////////////////////////////////////////////////
// The first device put on a tablet always control the system pointer.
// The mouse can be to set to keep the pointer at the same place so we have 
// to switch the mouse state (moveCursor) when a device is on the tablet.
// The sysTablet component (SysTablet.cpp) send a message we have to
// process so the process member is overriden.


/* includes *******************************************************************/

#include <Pointer.h>

#include <hsm/util/Debug.h>
#include <hsm/component/Point.h>
#include <hsm/component/ButtonPad.h>


namespace hsm {

/* Pointer instance ***********************************************************/

	const Pointer Pointer::instance;


/* Pointer implementation *****************************************************/

	OSStatus Pointer::handler(EventHandlerCallRef inRef, EventRef inEvent, void *userData) {
		static WindowPartCode windowPart;
		
		static ::Point mouse;
		static long wheelDelta;
		static EventMouseButton mouseButton;

		switch(GetEventKind(inEvent)) {
		case kEventMouseMoved :
		case kEventMouseDragged :
			GetEventParameter(inEvent, kEventParamMouseLocation, typeQDPoint, 0, sizeof(::Point), 0, &mouse);
			*(instance.position) = mouse.h, mouse.v;
			break;

		case kEventMouseWheelMoved :
			GetEventParameter(inEvent, kEventParamMouseWheelDelta, typeLongInteger, 0, sizeof(long), 0, &wheelDelta);
			*(instance.wheel)/Point::W += wheelDelta;
			break;

		case kEventMouseDown :
			// check if the click is inside the window
			GetEventParameter(inEvent, kEventParamWindowPartCode, typeWindowPartCode, 0, sizeof(WindowPartCode), 0, &windowPart);
			if(windowPart == inContent) {
				GetEventParameter(inEvent, kEventParamMouseButton, typeMouseButton, 0, sizeof(EventMouseButton), 0, &mouseButton);
				switch(mouseButton) {
				case kEventMouseButtonPrimary :
					*(instance.buttons)/ButtonPad::L = true;
					break;
				case kEventMouseButtonSecondary :
					*(instance.buttons)/ButtonPad::R = true;
					break;
				case kEventMouseButtonTertiary :
					*(instance.buttons)/ButtonPad::M = true;
					break;
				}
			}
			break;

		case kEventMouseUp :
			GetEventParameter(inEvent, kEventParamMouseButton, typeMouseButton, 0, sizeof(EventMouseButton), 0, &mouseButton);
			switch(mouseButton) {
			case kEventMouseButtonPrimary :
				*(instance.buttons)/ButtonPad::L = false;
				break;
			case kEventMouseButtonSecondary :
				*(instance.buttons)/ButtonPad::R = false;
				break;
			case kEventMouseButtonTertiary :
				*(instance.buttons)/ButtonPad::M = false;
				break;
			}
			break;
		} // switch(GetEventKind(inEvent))

		return eventNotHandledErr;
	}

	Pointer::Pointer() : impl::Component< EventHandler >("Pointer") {
		HSM_PRINT(1, "[Pointer::Pointer()]");

		nbEvents = 5;
		eventSpec = new Window::os::EventSpec[nbEvents];
		eventSpec[0].eventClass = kEventClassMouse;
		eventSpec[0].eventKind = kEventMouseUp;
		eventSpec[1].eventClass = kEventClassMouse;
		eventSpec[1].eventKind = kEventMouseDown;
		eventSpec[2].eventClass = kEventClassMouse;
		eventSpec[2].eventKind = kEventMouseWheelMoved;
		eventSpec[3].eventClass = kEventClassMouse;
		eventSpec[3].eventKind = kEventMouseMoved;
		eventSpec[4].eventClass = kEventClassMouse;
		eventSpec[4].eventKind = kEventMouseDragged;

		add(position = new Point("position", 2));
		add(wheel = new Point("wheel", 1));
		add(buttons = new ButtonPad("buttons", 3));
		
		hsm::repository::add(this);
	}

	Pointer::~Pointer() {
		HSM_PRINT(1, "[Pointer::~Pointer()]");

		hsm::repository::remove(this);
		delete[] eventSpec;
	}

	Window::os::Handler *Pointer::getHandler() const {
		return handler;
	}


} // namespace hsm
